@@ -2,7 +2,7 @@
 
 # Notification Center code borrowed from https://github.com/maranas/pyNotificationCenter/blob/master/pyNotificationCenter.py
 
-VERSION =   '1.1.4'
+VERSION =   '1.2.2'
 
 import os, sys, json, re, hashlib, argparse, urllib, time, base64, ConfigParser, gzip, mimetypes
 
@@ -107,19 +107,16 @@ parser.add_argument('environment', help = "Which environment to deploy to", narg
 parser.add_argument('-a', '--access-key', help = "AWS Access Key ID", type = str)
 parser.add_argument('-s', '--access-secret', help = "AWS Access Key Secret", type = str)
 parser.add_argument('-f', '--force', help = "Upload all files whether they are currently up to date on S3 or not", action = "store_true", default = False)
-parser.add_argument('--no-delete', help = "Don't remove orphaned files from S3", action = "store_true", default = False)
 parser.add_argument('--delete', help = "Remove orphaned files from S3", action = "store_true", default = False)
 parser.add_argument('--all', help = "Upload to all environments", action = "store_true", default = False)
 parser.add_argument('-n', '--dry-run', help = "Show which files would be updated without uploading to S3", action = "store_true", default = False)
 parser.add_argument('--acl', help = "The ACL to apply to uploaded files.", type = str, default = "public-read", choices = valid_acls)
 parser.add_argument('-v', '--version', help = "Print the script version and exit", action = "store_true", default = False)
 parser.add_argument('-z', '--gzip', help = "gzip files before uploading", action = "store_true", default = False)
+parser.add_argument('--confirm', help = "Confirm each file before deleting. Only works when --delete is set.", action = "store_true", default = False)
 
 args            =   parser.parse_args()
 
-if args.no_delete:
-    alert('--no-delete has been deprecated. Orphaned files will only be deleted if --delete is specified.')
-
 AWS_KEY         =   args.access_key
 AWS_SECRET      =   args.access_secret
 
@@ -206,16 +203,20 @@ def upload_files(env, config):
         keynames.append(keyname.lstrip('/'))
         s3key           =   s3bucket.get_key(keyname)
         local_file      =   open(filename, 'r')
-        md5             =   hashlib.md5()
-        is_gzipped      =   False
-        while True:
-            data        =   local_file.read(8192)
-            if not data:
-                break
-            if data.find('\x1f\x8b') == 0:
-                is_gzipped  =   True
-            md5.update(data)
-        if s3key is None or args.force or not s3key.etag.strip('"') == md5.hexdigest():
+        md5             =   boto.utils.compute_md5(local_file)[0] # this needs to be computed before gzipping
+        local_file.close()
+        if args.gzip or config.get('gzip', False):
+            if not mimetypes.guess_type(filename)[1] == 'gzip':
+                f_in    =   open(filename, 'rb')
+                f_out   =   gzip.open(filename + '.gz', 'wb')
+                f_out.writelines(f_in)
+                f_out.close()
+                f_in.close()
+                filename    =   f_out.name
+        local_file      =   open(filename, 'r')
+        is_gzipped      =   local_file.read().find('\x1f\x8b') == 0
+        local_file.seek(0)
+        if s3key is None or args.force or not s3key.get_metadata('d3ploy-hash') == md5:
             alert('Copying %s to %s%s' % (filename, bucket, keyname))
             updated     +=  1
             if args.dry_run:
@@ -223,33 +224,35 @@ def upload_files(env, config):
             if s3key is None:
                 s3key   =   s3bucket.new_key(keyname)
             headers     =   {}
-            if is_gzipped or args.gzip or config.get('gzip', False):
+            if is_gzipped or mimetypes.guess_type(filename)[1] == 'gzip':
                 headers['Content-Encoding'] =   'gzip'
-            if args.gzip and not mimetypes.guess_type(filename)[1] == 'gzip':
-                f_in    =   open(filename, 'rb')
-                f_out   =   gzip.open(filename + '.gz', 'wb')
-                f_out.writelines(f_in)
-                f_out.close()
-                f_in.close()
-                filename    =   f_out.name
-            s3key.set_contents_from_filename(filename, headers = headers)
+            s3key.set_metadata('d3ploy-hash', md5)
+            s3key.set_contents_from_file(local_file, headers = headers)
             s3key.set_acl(args.acl)
-            if not filename in files:
-                # this filename was modified by gzipping
-                os.remove(filename)
+        if not filename in files:
+            # this filename was modified by gzipping
+            os.remove(filename)
+        local_file.close()
 
-    if args.delete:
+    if args.delete or config.get('delete', False):
         for key in s3bucket.list(prefix = bucket_path.lstrip('/')):
             if not key.name in keynames:
-                alert('Deleting %s/%s' % (bucket, key.name.lstrip('/')))
-                deleted     +=  1
-                if args.dry_run:
-                    continue
-                key.delete()
+                if args.confirm or config.get('confirm', False):
+                    confirmed   =   raw_input('Remove %s/%s [yN]: ' % (bucket, key.name.lstrip('/'))) in ["Y", "y"]
+                else:
+                    confirmed   =   True
+                if confirmed:
+                    alert('Deleting %s/%s' % (bucket, key.name.lstrip('/')))
+                    deleted     +=  1
+                    if args.dry_run:
+                        continue
+                    key.delete()
+                else:
+                    alert('Skipping removal of %s/%s' % (bucket, key.name.lstrip('/')))
         
     verb    =   "would be" if args.dry_run else "were"
     notify(args.environment, "%d files %s updated" % (updated, verb))
-    if args.delete:
+    if args.delete or config.get('delete', False):
         notify(args.environment, "%d files %s removed" % (deleted, verb))
     alert("")
 
